import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { Subject } from 'rxjs/Subject';
import { isBrowser, isURL } from './misc/Util';
import { Channel } from './service/Channel';
var CONNECT_TIMEOUT_FOR_NODE = 3000;
var listenSubject = new BehaviorSubject('');
/**
 * Service class responsible to establish connections between peers via
 * `WebSocket`.
 */
var WebSocketBuilder = (function () {
    function WebSocketBuilder(wc) {
        this.wc = wc;
        this.channelsSubject = new Subject();
    }
    WebSocketBuilder.listen = function () {
        return listenSubject;
    };
    WebSocketBuilder.newIncomingSocket = function (wc, ws, senderId) {
        wc.webSocketBuilder.channelsSubject.next(new Channel(wc, ws, { id: senderId }));
    };
    Object.defineProperty(WebSocketBuilder.prototype, "onChannel", {
        get: function () {
            return this.channelsSubject.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Establish `WebSocket` with a server.
     *
     * @param url Server url
     */
    WebSocketBuilder.prototype.connect = function (url) {
        return new Promise(function (resolve, reject) {
            try {
                if (isURL(url) && url.search(/^wss?/) !== -1) {
                    var ws_1 = new global.WebSocket(url);
                    ws_1.onopen = function () { return resolve(ws_1); };
                    ws_1.onclose = function (closeEvt) { return reject(new Error("WebSocket connection to '" + url + "' failed with code " + closeEvt.code + ": " + closeEvt.reason)); };
                    if (!isBrowser) {
                        // Timeout for node (otherwise it will loop forever if incorrect address)
                        setTimeout(function () {
                            if (ws_1.readyState !== ws_1.OPEN) {
                                reject(new Error("WebSocket " + CONNECT_TIMEOUT_FOR_NODE + "ms connection timeout with " + url));
                            }
                        }, CONNECT_TIMEOUT_FOR_NODE);
                    }
                }
                else {
                    throw new Error(url + " is not a valid URL");
                }
            }
            catch (err) {
                console.error('WebSocketBuilder ERROR');
                reject(err);
            }
        });
    };
    /**
     * Establish a `Channel` with a server peer identified by `id`.
     *
     * @param url Server url
     * @param id  Peer id
     */
    WebSocketBuilder.prototype.connectTo = function (url, id) {
        var _this = this;
        var fullUrl = url + "/internalChannel?wcId=" + this.wc.id + "&senderId=" + this.wc.myId;
        return new Promise(function (resolve, reject) {
            if (isURL(url) && url.search(/^wss?/) !== -1) {
                var ws_2 = new global.WebSocket(fullUrl);
                var channel_1 = new Channel(_this.wc, ws_2, { id: id });
                ws_2.onopen = function () { return resolve(channel_1); };
                ws_2.onclose = function (closeEvt) { return reject(new Error("WebSocket connection to '" + url + "' failed with code " + closeEvt.code + ": " + closeEvt.reason)); };
                if (!isBrowser) {
                    // Timeout for node (otherwise it will loop forever if incorrect address)
                    setTimeout(function () {
                        if (ws_2.readyState !== ws_2.OPEN) {
                            reject(new Error("WebSocket " + CONNECT_TIMEOUT_FOR_NODE + "ms connection timeout with " + url));
                        }
                    }, CONNECT_TIMEOUT_FOR_NODE);
                }
            }
            else {
                throw new Error(url + " is not a valid URL");
            }
        });
    };
    return WebSocketBuilder;
}());
export { WebSocketBuilder };
