import './misc/polyfills.browser';
import { TopologyEnum } from './service/topology/Topology';
import { SignalingState as SigState } from './Signaling';
export { WebGroup, WebGroupState } from './WebChannelFacade';
import { enableLog as debug } from './misc/Util';
/**
 * Enable/Disable console logs. By default the logs are disabled. Consol logs
 * group and signaling states, connection establishment, disconnection and
 * other useful information for debugging.
 * @param {boolean} value
 */
export function enableLog(value) { debug(value); }
/**
 * The state enum of the signaling server for WebRTC.
 */
var SignalingState = (function () {
    function SignalingState() {
    }
    Object.defineProperty(SignalingState, "CONNECTING", {
        /**
         * The connection is not yet open.
         * @type {number}
         */
        get: function () { return SigState.CONNECTING; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, SignalingState.CONNECTING, {
        /**
         * Equals to `'CONNECTING'`.
         * @type {string}
         */
        get: function () { return SigState[SigState.CONNECTING]; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, "OPEN", {
        /**
         * The connection is open and ready to communicate.
         * @type {number}
         */
        get: function () { return SigState.OPEN; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, SignalingState.OPEN, {
        /**
         * Equals to `'OPEN'`.
         * @type {string}
         */
        get: function () { return SigState[SigState.OPEN]; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, "CONNECTED_WITH_FIRST_MEMBER", {
        /**
         * `RTCDataChannel` has been established with one of the group member.
         * From now the signaling is no longer needed, because the joining process
         * will continue with a help of this member.
         * @type {number}
         */
        get: function () { return SigState.CONNECTED_WITH_FIRST_MEMBER; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, SignalingState.CONNECTED_WITH_FIRST_MEMBER, {
        /**
         * Equals to `'CONNECTED_WITH_FIRST_MEMBER'`.
         * @type {string}
         */
        get: function () {
            return SigState[SigState.CONNECTED_WITH_FIRST_MEMBER];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, "READY_TO_JOIN_OTHERS", {
        /**
         * You has successfully been joined a web group and ready to help join others.
         * @type {number}
         */
        get: function () { return SigState.READY_TO_JOIN_OTHERS; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, SignalingState.READY_TO_JOIN_OTHERS, {
        /**
         * Equals to `'READY_TO_JOIN_OTHERS'`.
         * @type {string}
         */
        get: function () { return SigState[SigState.READY_TO_JOIN_OTHERS]; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, "CLOSED", {
        /**
         * The connection is closed.
         * @type {number}
         */
        get: function () { return SigState.CLOSED; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SignalingState, SignalingState.CLOSED, {
        /**
         * Equals to `'CLOSED'`.
         * @type {string}
         */
        get: function () { return SigState[SigState.CLOSED]; },
        enumerable: true,
        configurable: true
    });
    return SignalingState;
}());
export { SignalingState };
/**
 * The topology enum.
 */
var Topology = (function () {
    function Topology() {
    }
    Object.defineProperty(Topology, "FULL_MESH", {
        /**
         * Full mesh topology identifier.
         * @type {number}
         */
        get: function () { return TopologyEnum.FULL_MESH; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Topology, Topology.FULL_MESH, {
        /**
         * Equals to `'FULL_MESH'`.
         * @type {string}
         */
        get: function () { return TopologyEnum[TopologyEnum.FULL_MESH]; },
        enumerable: true,
        configurable: true
    });
    return Topology;
}());
export { Topology };
/**
 * The options to be passed into {@link WebGroup} constructor.
 * @typedef {Object} WebGroupOptions
 * @property {Topology} [topology] Topology identifier
 * (Full mesh is the only one supported by Netflux for now).
 * @property {string} [signalingURL] Signaling URL for WebRTC.
 * @property {RTCIceServer[]} [iceServers] Array of Ice servers for WebRTC.
 * @property {boolean} [autoRejoin] Whether to automatically rejoin the web group
 * on disconnect or not. Its value may be modified after {@link WebGroup}
 * instantiation at any time.
 */
/**
 * The options to be passed into {@link WebGroupBotServer} constructor.
 * @typedef {Object} WebGroupBotServerOptions
 * @property {Topology} [topology] See WebGroupOptions.topology
 * @property {string} [signalingURL] See WebGroupOptions.signalingURL
 * @property {RTCIceServer[]} [iceServers] See WebGroupOptions.iceServers
 * @property {boolean} [autoRejoin] See WebGroupOptions.autoRejoin
 * @property {Object} bot Server related options of the bot.
 * @property {NodeJSHttpServer|NodeJSHttpsServer} bot.server NodeJS http(s) server.
 * @property {string} [bot.url] Bot server URL.
 * @property {boolean} [bot.perMessageDeflate] Enable/disable permessage-deflate.
 */
/**
 * @external {RTCIceServer} https://developer.mozilla.org/en/docs/Web/API/RTCIceServer
 */
/**
 * @external {Uint8Array} https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Global_Objects/Uint8Array
 */
/**
 * @external {NodeJSHttpServer} https://nodejs.org/api/http.html#http_class_http_server
 */
/**
 * @external {NodeJSHttpsServer} https://nodejs.org/api/https.html#https_class_https_server
 */
