/**
 * Equals to true in any browser.
 */
export var isBrowser = (typeof global.window === 'undefined') ? false : true;
/**
 * Equals to true in Firefox and false elsewhere.
 * Thanks to https://github.com/lancedikson/bowser
 */
export var isFirefox = (isBrowser &&
    navigator !== undefined &&
    navigator.userAgent !== undefined &&
    /firefox|iceweasel|fxios/i.test(navigator.userAgent)) ? true : false;
/**
 * Check whether the string is a valid URL.
 */
export function isURL(str) {
    var regex = '^' +
        // protocol identifier
        '(?:wss|ws)://' +
        // Host name/IP
        '[^\\s]+' +
        // port number
        '(?::\\d{2,5})?' +
        '$';
    return (new RegExp(regex, 'i')).test(str);
}
/**
 * Generate random key which will be used to join the network.
 */
export function generateKey() {
    var mask = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
    var length = 42; // Should be less then MAX_KEY_LENGTH value
    var values = randNumbers(length);
    var result = '';
    for (var i = 0; i < length; i++) {
        result += mask[values[i] % mask.length];
    }
    return result;
}
export function randNumbers(length) {
    if (length === void 0) { length = 1; }
    var res;
    if (isBrowser) {
        res = new Uint32Array(length);
        global.crypto.getRandomValues(res);
    }
    else {
        res = [];
        var bytes = crypto.randomBytes(4 * length);
        for (var i = 0; i < bytes.length; i += 4) {
            res[res.length] = bytes.readUInt32BE(i, true);
        }
    }
    return res;
}
export var MAX_KEY_LENGTH = 512;
var netfluxCSS = 'background-color: #FFCA28; padding: 0 3px';
var signalingStateCSS = 'background-color: #9FA8DA; padding: 0 2px';
var webGroupStateCSS = 'background-color: #EF9A9A; padding: 0 2px';
var log;
export function enableLog(isDebug) {
    if (isDebug) {
        log = {
            info: function (msg) {
                var rest = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    rest[_i - 1] = arguments[_i];
                }
                if (rest.length === 0) {
                    console.info("%cNETFLUX%c: " + msg, netfluxCSS, '');
                }
                else {
                    console.info("%cNETFLUX%c: " + msg, rest, netfluxCSS, '');
                }
            },
            signalingState: function (msg) {
                console.info("%cNETFLUX%c: Signaling: %c" + msg + "%c", netfluxCSS, '', signalingStateCSS, '');
            },
            webGroupState: function (msg) {
                console.info("%cNETFLUX%c: WebGroup: %c" + msg + "%c", netfluxCSS, '', webGroupStateCSS, '');
            },
        };
    }
    else {
        log = {
            info: function () { },
            signalingState: function () { },
            webGroupState: function () { },
        };
    }
}
enableLog(false);
export { log };
