var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { isBrowser, isFirefox, log } from '../misc/Util';
import { channel as channelProto } from '../proto';
import { Service } from './Service';
/**
 * Service id.
 */
var ID = 400;
var WEBSOCKET_PING_TIMEOUT = 3000;
var WEBSOCKET_PING_START_DELAY = 2000;
/* Preconstructed messages */
var pingMsg = Service.encodeServiceMessage(ID, channelProto.Message.encode(channelProto.Message.create({ ping: true })).finish());
var pongMsg = Service.encodeServiceMessage(ID, channelProto.Message.encode(channelProto.Message.create({ pong: true })).finish());
/**
 * Wrapper class for `RTCDataChannel` and `WebSocket`.
 */
var Channel = (function (_super) {
    __extends(Channel, _super);
    /**
     * Creates a channel from existing `RTCDataChannel` or `WebSocket`.
     */
    function Channel(wc, connection, options) {
        if (options === void 0) { options = { id: -1 }; }
        var _this = _super.call(this, ID, channelProto.Message, wc.serviceMessageSubject) || this;
        log.info("new connection: Me: " + wc.myId + " with " + options.id);
        _this.wc = wc;
        _this.connection = connection;
        _this.id = options.id;
        _this.rtcPeerConnection = options.rtcPeerConnection;
        _this.isIntermediary = false;
        // Configure `send` function
        if (isBrowser) {
            connection.binaryType = 'arraybuffer';
            _this.send = _this.sendInBrowser;
        }
        else if (!_this.rtcPeerConnection) {
            _this.send = _this.sendInNodeViaWebSocket;
        }
        else {
            connection.binaryType = 'arraybuffer';
            _this.send = _this.sendInNodeViaDataChannel;
        }
        _this.onClose = function (evt) {
            log.info("Connection with " + _this.id + " has closed");
            _this.connection.onclose = function () { };
            _this.connection.onmessage = function () { };
            _this.connection.onerror = function () { };
            wc.topologyService.onChannelClose(evt, _this);
            if (_this.rtcPeerConnection && _this.rtcPeerConnection.signalingState !== 'closed') {
                _this.rtcPeerConnection.close();
            }
        };
        // Configure handlers
        _this.connection.onmessage = function (_a) {
            var data = _a.data;
            return wc.onMessageProxy(_this, new Uint8Array(data));
        };
        _this.connection.onclose = function (evt) { return _this.onClose(evt); };
        _this.connection.onerror = function (evt) { return wc.topologyService.onChannelError(evt, _this); };
        // Start ping interval for WebSocket
        if (!_this.rtcPeerConnection) {
            _this.onServiceMessage.subscribe(function (_a) {
                var msg = _a.msg;
                if (msg.ping) {
                    _this.send(_this.wc.encode({ recipientId: _this.id, content: pongMsg }));
                }
                else if (msg.pong) {
                    _this.pongReceived = true;
                }
                else {
                    log.warn('Unknown message from Channel service', msg);
                }
            });
            global.setTimeout(function () {
                _this.ping();
            }, WEBSOCKET_PING_START_DELAY);
        }
        return _this;
    }
    Channel.prototype.markAsIntermediry = function () {
        this.wc.topologyService.initIntermediary(this);
    };
    Channel.prototype.close = function () {
        if (this.connection.readyState !== 'closed' &&
            this.connection.readyState !== 'closing' &&
            this.connection.readyState !== WebSocket.CLOSED &&
            this.connection.readyState !== WebSocket.CLOSING) {
            log.info("I:" + this.wc.myId + " close connection with " + this.id);
            this.connection.close();
            if (isFirefox && this.rtcPeerConnection && this.rtcPeerConnection.signalingState !== 'closed') {
                this.onClose(new global.Event('close'));
            }
        }
    };
    Channel.prototype.closeQuietly = function () {
        this.connection.onmessage = undefined;
        this.connection.onclose = undefined;
        this.connection.onerror = undefined;
        if (this.rtcPeerConnection) {
            this.rtcPeerConnection.oniceconnectionstatechange = undefined;
        }
        log.info("I:" + this.wc.myId + " close QUIETLY connection with " + this.id);
        this.connection.close();
        if (this.rtcPeerConnection && this.rtcPeerConnection.signalingState !== 'closed') {
            this.rtcPeerConnection.close();
        }
    };
    Channel.prototype.sendInBrowser = function (data) {
        // if (this.connection.readyState !== 'closed' && new Int8Array(data).length !== 0) {
        if (this.isOpen()) {
            try {
                this.connection.send(data);
            }
            catch (err) {
                console.error("Channel send: " + err.message);
            }
        }
    };
    Channel.prototype.sendInNodeViaWebSocket = function (data) {
        if (this.isOpen()) {
            try {
                this.connection.send(data, { binary: true });
            }
            catch (err) {
                console.error("Channel send: " + err.message);
            }
        }
    };
    Channel.prototype.sendInNodeViaDataChannel = function (data) {
        this.sendInBrowser(data.slice(0));
    };
    Channel.prototype.isOpen = function () {
        return this.connection.readyState === WebSocket.OPEN || this.connection.readyState === 'open';
    };
    Channel.prototype.ping = function () {
        var _this = this;
        this.pongReceived = false;
        if (this.isOpen()) {
            this.send(this.wc.encode({ recipientId: this.id, content: pingMsg }));
            global.setTimeout(function () {
                if (_this.isOpen()) {
                    if (_this.pongReceived) {
                        _this.ping();
                    }
                    else {
                        _this.close();
                    }
                }
            }, WEBSOCKET_PING_TIMEOUT);
        }
    };
    return Channel;
}(Service));
export { Channel };
