var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { Subject } from 'rxjs/Subject';
import { channelBuilder } from '../proto';
import { WebSocketBuilder } from '../WebSocketBuilder';
import { Service } from './Service';
import { WebRTCBuilder } from './WebRTCBuilder';
/**
 * Service id.
 */
var ID = 100;
var ME = {
    wsUrl: '',
    isWrtcSupport: false,
};
var request;
var response;
/**
 * It is responsible to build a channel between two peers with a help of `WebSocketBuilder` and `WebRTCBuilder`.
 * Its algorithm determine which channel (socket or dataChannel) should be created
 * based on the services availability and peers' preferences.
 */
var ChannelBuilder = (function (_super) {
    __extends(ChannelBuilder, _super);
    function ChannelBuilder(wc) {
        var _this = _super.call(this, ID, channelBuilder.Message, wc.serviceMessageSubject) || this;
        _this.wc = wc;
        _this.pendingRequests = new Map();
        _this.channelsSubject = new Subject();
        // Listen on Channels as RTCDataChannels if WebRTC is supported
        ME.isWrtcSupport = WebRTCBuilder.isSupported;
        if (ME.isWrtcSupport) {
            wc.webRTCBuilder.onChannelFromWebChannel()
                .subscribe(function (ch) { return _this.handleChannel(ch); });
        }
        // Listen on Channels as WebSockets if the peer is listening on WebSockets
        WebSocketBuilder.listen().subscribe(function (url) {
            ME.wsUrl = url;
            if (url) {
                wc.webSocketBuilder.onChannel.subscribe(function (ch) { return _this.handleChannel(ch); });
            }
            // Update preconstructed messages (for performance only)
            var content = { wsUrl: url, isWrtcSupport: ME.isWrtcSupport };
            request = _super.prototype.encode.call(_this, { request: content });
            response = _super.prototype.encode.call(_this, { response: content });
        });
        // Subscribe to WebChannel internal messages
        _this.onServiceMessage.subscribe(function (msg) { return _this.treatServiceMessage(msg); });
        return _this;
    }
    Object.defineProperty(ChannelBuilder.prototype, "onChannel", {
        get: function () {
            return this.channelsSubject.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Establish a `Channel` with the peer identified by `id`.
     */
    ChannelBuilder.prototype.connectTo = function (id) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.pendingRequests.set(id, {
                resolve: function (ch) {
                    _this.pendingRequests.delete(id);
                    resolve(ch);
                }, reject: function (err) {
                    _this.pendingRequests.delete(id);
                    reject(err);
                },
            });
            _this.wc.sendToProxy({ recipientId: id, content: request });
        });
    };
    ChannelBuilder.prototype.handleChannel = function (ch) {
        var pendReq = this.pendingRequests.get(ch.id);
        if (pendReq) {
            pendReq.resolve(ch);
        }
        else {
            this.channelsSubject.next(ch);
        }
    };
    ChannelBuilder.prototype.treatServiceMessage = function (_a) {
        var _this = this;
        var channel = _a.channel, senderId = _a.senderId, recipientId = _a.recipientId, msg = _a.msg;
        switch (msg.type) {
            case 'failed': {
                console.warn('treatServiceMessage ERROR: ', msg.failed);
                var pr = this.pendingRequests.get(senderId);
                if (pr !== undefined) {
                    pr.reject(new Error(msg.failed));
                }
                break;
            }
            case 'request': {
                var _b = msg.request, wsUrl = _b.wsUrl, isWrtcSupport = _b.isWrtcSupport;
                // If remote peer is listening on WebSocket, connect to him
                if (wsUrl) {
                    this.wc.webSocketBuilder.connectTo(wsUrl, senderId)
                        .then(function (ch) { return _this.handleChannel(ch); })
                        .catch(function (reason) {
                        if (ME.wsUrl) {
                            // Ask him to connect to me via WebSocket
                            _this.wc.sendToProxy({ recipientId: senderId, content: response });
                        }
                        else {
                            // Send failed reason
                            _this.wc.sendToProxy({
                                recipientId: senderId,
                                content: _super.prototype.encode.call(_this, { failed: "Failed to establish a socket: " + reason }),
                            });
                        }
                    });
                    // If remote peer is able to connect over RTCDataChannel, verify first if I am listening on WebSocket
                }
                else if (isWrtcSupport) {
                    if (ME.wsUrl) {
                        // Ask him to connect to me via WebSocket
                        this.wc.sendToProxy({ recipientId: senderId, content: response });
                    }
                    else if (ME.isWrtcSupport) {
                        this.wc.webRTCBuilder.connectOverWebChannel(senderId)
                            .then(function (ch) { return _this.handleChannel(ch); })
                            .catch(function (reason) {
                            // Send failed reason
                            _this.wc.sendToProxy({
                                recipientId: senderId,
                                content: _super.prototype.encode.call(_this, { failed: "Failed establish a data channel: " + reason }),
                            });
                        });
                    }
                    else {
                        // Send failed reason
                        this.wc.sendToProxy({
                            recipientId: senderId,
                            content: _super.prototype.encode.call(this, { failed: 'No common connectors' }),
                        });
                    }
                    // If peer is not listening on WebSocket and is not able to connect over RTCDataChannel
                }
                else if (!wsUrl && !isWrtcSupport) {
                    if (ME.wsUrl) {
                        // Ask him to connect to me via WebSocket
                        this.wc.sendToProxy({ recipientId: senderId, content: response });
                    }
                    else {
                        // Send failed reason
                        this.wc.sendToProxy({
                            recipientId: senderId,
                            content: _super.prototype.encode.call(this, { failed: 'No common connectors' }),
                        });
                    }
                }
                break;
            }
            case 'response': {
                var wsUrl = msg.response.wsUrl;
                if (wsUrl) {
                    this.wc.webSocketBuilder.connectTo(wsUrl, senderId)
                        .then(function (ch) { return _this.handleChannel(ch); })
                        .catch(function (reason) {
                        _this.pendingRequests.get(senderId)
                            .reject(new Error("Failed to establish a socket: " + reason));
                    });
                }
                break;
            }
        }
    };
    return ChannelBuilder;
}(Service));
export { ChannelBuilder };
