var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
import { Observable } from 'rxjs/Observable';
import { filter, map } from 'rxjs/operators';
import { ReplaySubject } from 'rxjs/ReplaySubject';
import { log } from '../misc/Util';
import { webRTCBuilder } from '../proto';
import { Channel } from './Channel';
import { Service } from './Service';
/**
 * Service id.
 */
var ID = 300;
/**
 * Service class responsible to establish `RTCDataChannel` between two clients via
 * signaling server or `WebChannel`.
 *
 */
var WebRTCBuilder = (function (_super) {
    __extends(WebRTCBuilder, _super);
    function WebRTCBuilder(wc, iceServers) {
        var _this = _super.call(this, ID, webRTCBuilder.Message, wc.serviceMessageSubject) || this;
        _this.wc = wc;
        _this.rtcConfiguration = { iceServers: iceServers };
        _this.clients = new Map();
        return _this;
    }
    Object.defineProperty(WebRTCBuilder, "isSupported", {
        /**
         * Indicates whether WebRTC is supported by the environment.
         */
        get: function () {
            return global.RTCPeerConnection !== undefined;
        },
        enumerable: true,
        configurable: true
    });
    WebRTCBuilder.prototype.onChannelFromWebChannel = function () {
        var _this = this;
        if (WebRTCBuilder.isSupported) {
            return this.onChannel(this.onServiceMessage.pipe(filter(function (_a) {
                var msg = _a.msg;
                return msg.isInitiator;
            }), map(function (_a) {
                var msg = _a.msg, senderId = _a.senderId;
                msg.id = senderId;
                return msg;
            })), function (msg, id) { return _this.wc.sendToProxy({ recipientId: id, content: _super.prototype.encode.call(_this, msg) }); });
        }
        throw new Error('WebRTC is not supported');
    };
    /**
     * Establish an `RTCDataChannel` with a peer identified by `id` trough `WebChannel`.
     * Starts by sending an **SDP offer**.
     *
     * @param id  Peer id
     */
    WebRTCBuilder.prototype.connectOverWebChannel = function (id) {
        var _this = this;
        if (WebRTCBuilder.isSupported) {
            return this.establishChannel(this.onServiceMessage.pipe(filter(function (_a) {
                var msg = _a.msg, senderId = _a.senderId;
                return senderId === id && !msg.isInitiator;
            }), map(function (_a) {
                var msg = _a.msg;
                return ({ answer: msg.answer, iceCandidate: msg.iceCandidate });
            })), function (msg) {
                msg.isInitiator = true;
                _this.wc.sendToProxy({ recipientId: id, content: _super.prototype.encode.call(_this, msg) });
            }, id);
        }
        throw new Error('WebRTC is not supported');
    };
    /**
     * Listen on `RTCDataChannel` from Signaling server.
     * Starts to listen on **SDP answer**.
     */
    WebRTCBuilder.prototype.onChannelFromSignaling = function (signalingConnection) {
        var _this = this;
        if (WebRTCBuilder.isSupported) {
            return this.onChannel(signalingConnection.onMessage.pipe(filter(function (_a) {
                var id = _a.id;
                return id !== 0;
            }), map(function (msg) {
                if (msg.type === 'data') {
                    var completeData = _super.prototype.decode.call(_this, msg.data);
                    completeData.id = msg.id;
                    return completeData;
                }
                else {
                    return { isError: true };
                }
            })), function (msg, id) {
                var bytes = webRTCBuilder.Message
                    .encode(webRTCBuilder.Message.create(msg))
                    .finish();
                var isEnd = msg.iceCandidate !== undefined && msg.iceCandidate.candidate === '';
                signalingConnection.send({ id: id, isEnd: isEnd, data: bytes });
            });
        }
        throw new Error('WebRTC is not supported');
    };
    /**
     * Establish an `RTCDataChannel` with a peer identified by `id` trough Signaling server.
     * Starts by sending an **SDP offer**.
     */
    WebRTCBuilder.prototype.connectOverSignaling = function (signalingConnection) {
        var _this = this;
        if (WebRTCBuilder.isSupported) {
            return this.establishChannel(signalingConnection.onMessage.pipe(filter(function (_a) {
                var id = _a.id;
                return id === 0;
            }), map(function (msg) {
                return msg.type === 'data' ? _super.prototype.decode.call(_this, msg.data) : { isError: true };
            })), function (msg) {
                var bytes = webRTCBuilder.Message
                    .encode(webRTCBuilder.Message.create(msg))
                    .finish();
                var isEnd = msg.iceCandidate !== undefined && msg.iceCandidate.candidate === '';
                signalingConnection.send({ isEnd: isEnd, data: bytes });
            });
        }
        throw new Error('WebRTC is not supported');
    };
    WebRTCBuilder.prototype.establishChannel = function (onMessage, send, id) {
        var _this = this;
        if (id === void 0) { id = 1; }
        var pc = new global.RTCPeerConnection(this.rtcConfiguration);
        var remoteCandidateStream = new ReplaySubject();
        this.localCandidates(pc).subscribe(function (iceCandidate) { return send({ iceCandidate: iceCandidate }); }, function (err) { return console.warn(err); }, function () { return send({ iceCandidate: { candidate: '' } }); });
        return new Promise(function (resolve, reject) {
            var subs = onMessage.subscribe(function (_a) {
                var answer = _a.answer, iceCandidate = _a.iceCandidate, isError = _a.isError;
                if (answer) {
                    pc.setRemoteDescription({ type: 'answer', sdp: answer })
                        .then(function () {
                        remoteCandidateStream.subscribe(function (ic) {
                            pc.addIceCandidate(new global.RTCIceCandidate(ic))
                                .catch(reject);
                        }, function (err) { return console.warn(err); }, function () { return subs.unsubscribe(); });
                    })
                        .catch(reject);
                }
                else if (iceCandidate) {
                    if (iceCandidate.candidate !== '') {
                        remoteCandidateStream.next(iceCandidate);
                    }
                    else {
                        remoteCandidateStream.complete();
                    }
                }
                else if (isError) {
                    reject(new Error('Remote peer no longer available via Signaling'));
                }
                else {
                    reject(new Error('Unknown message from a remote peer'));
                }
            }, function (err) { return reject(err); }, function () { return reject(new Error('Failed to establish RTCDataChannel: the connection with Signaling server was closed')); });
            _this.openChannel(pc, id)
                .then(resolve)
                .catch(reject);
            pc.createOffer()
                .then(function (offer) { return pc.setLocalDescription(offer); })
                .then(function () { return send({ offer: pc.localDescription.sdp }); })
                .catch(reject);
        });
    };
    WebRTCBuilder.prototype.onChannel = function (onMessage, send) {
        var _this = this;
        return Observable.create(function (observer) {
            onMessage.subscribe(function (_a) {
                var offer = _a.offer, iceCandidate = _a.iceCandidate, id = _a.id, isError = _a.isError;
                var client = _this.clients.get(id);
                var pc;
                var remoteCandidateStream;
                if (client) {
                    _b = __read(client, 2), pc = _b[0], remoteCandidateStream = _b[1];
                }
                else {
                    pc = new global.RTCPeerConnection(_this.rtcConfiguration);
                    remoteCandidateStream = new ReplaySubject();
                    _this.localCandidates(pc).subscribe(function (ic) { return send({ iceCandidate: ic }, id); }, function (err) { return console.warn(err); }, function () { return send({ iceCandidate: { candidate: '' } }, id); });
                    _this.clients.set(id, [pc, remoteCandidateStream]);
                }
                if (offer) {
                    _this.openChannel(pc)
                        .then(function (ch) { return observer.next(ch); })
                        .catch(function (err) {
                        _this.clients.delete(id);
                        console.warn("Client \"" + id + "\" failed to establish RTCDataChannel with you: " + err.message);
                    });
                    pc.setRemoteDescription({ type: 'offer', sdp: offer })
                        .then(function () { return remoteCandidateStream.subscribe(function (ic) {
                        pc.addIceCandidate(new global.RTCIceCandidate(ic))
                            .catch(function (err) { return console.warn(err); });
                    }, function (err) { return console.warn(err); }, function () { return _this.clients.delete(id); }); })
                        .then(function () { return pc.createAnswer(); })
                        .then(function (answer) { return pc.setLocalDescription(answer); })
                        .then(function () { return send({ answer: pc.localDescription.sdp }, id); })
                        .catch(function (err) {
                        _this.clients.delete(id);
                        console.error(err);
                    });
                }
                else if (iceCandidate) {
                    if (iceCandidate.candidate !== '') {
                        remoteCandidateStream.next(iceCandidate);
                    }
                    else {
                        remoteCandidateStream.complete();
                    }
                }
                else if (isError) {
                    console.warn('Remote peer no longer available via Signaling');
                }
                else {
                    console.error(new Error('Unknown message from a remote peer'));
                }
                var _b;
            }, function (err) { return observer.error(err); }, function () { return observer.complete(); });
        });
    };
    WebRTCBuilder.prototype.localCandidates = function (pc) {
        return Observable.create(function (observer) {
            pc.onicecandidate = function (evt) {
                if (evt.candidate !== null) {
                    observer.next({
                        candidate: evt.candidate.candidate,
                        sdpMid: evt.candidate.sdpMid,
                        sdpMLineIndex: evt.candidate.sdpMLineIndex,
                    });
                }
                else {
                    observer.complete();
                }
            };
        });
    };
    WebRTCBuilder.prototype.openChannel = function (pc, id) {
        var _this = this;
        if (id !== undefined) {
            try {
                var dc_1 = pc.createDataChannel((this.wc.myId).toString());
                var channel_1 = new Channel(this.wc, dc_1, { rtcPeerConnection: pc, id: id });
                return new Promise(function (resolve, reject) {
                    pc.oniceconnectionstatechange = function () {
                        if (pc.iceConnectionState === 'failed') {
                            reject('Failed to establish PeerConnection: ' +
                                'The ICE candidate did not find compatible matches for all components of the connection');
                        }
                    };
                    dc_1.onopen = function () {
                        pc.oniceconnectionstatechange = function () {
                            log.info("ICE connection state with " + channel_1.id + " changed to " + pc.iceConnectionState.toUpperCase());
                            if (pc.iceConnectionState === 'failed') {
                                channel_1.close();
                            }
                        };
                        resolve(channel_1);
                    };
                });
            }
            catch (err) {
                return Promise.reject(err);
            }
        }
        else {
            return new Promise(function (resolve, reject) {
                pc.oniceconnectionstatechange = function () {
                    if (pc.iceConnectionState === 'failed') {
                        reject('The ICE candidate did not find compatible matches for all components of the connection');
                    }
                };
                pc.ondatachannel = function (dcEvt) {
                    var dc = dcEvt.channel;
                    var peerId = Number.parseInt(dc.label, 10);
                    var channel = new Channel(_this.wc, dc, { rtcPeerConnection: pc, id: peerId });
                    dc.onopen = function (evt) {
                        pc.oniceconnectionstatechange = function () {
                            log.info("ICE connection state with " + channel.id + " changed to " + pc.iceConnectionState.toUpperCase());
                            if (pc.iceConnectionState === 'failed') {
                                channel.close();
                            }
                        };
                        resolve(channel);
                    };
                };
            });
        }
    };
    return WebRTCBuilder;
}(Service));
export { WebRTCBuilder };
