var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
import { log } from '../../misc/Util';
import { fullMesh } from '../../proto';
import { Service } from '../Service';
/**
 * {@link FullMesh} identifier.
 * @ignore
 * @type {number}
 */
export var FULL_MESH = 3;
var MAX_JOIN_ATTEMPTS = 100;
/**
 * Fully connected web channel manager. Implements fully connected topology
 * network, when each peer is connected to each other.
 *
 */
var FullMesh = (function (_super) {
    __extends(FullMesh, _super);
    function FullMesh(wc) {
        var _this = _super.call(this, FULL_MESH, fullMesh.Message, wc.serviceMessageSubject) || this;
        _this.wc = wc;
        _this.channels = new Set();
        _this.jps = new Map();
        _this.joinAttempts = 0;
        _this.intermediaryChannel = undefined;
        _this.joinSucceedContent = _super.prototype.encode.call(_this, { joinSucceed: true });
        _this.onServiceMessage.subscribe(function (msg) { return _this.handleSvcMsg(msg); });
        _this.wc.channelBuilder.onChannel.subscribe(function (ch) { return _this.peerJoined(ch); });
        return _this;
    }
    FullMesh.prototype.clean = function () { };
    FullMesh.prototype.addJoining = function (ch, members) {
        log.info("FULL_MESH: I:" + this.wc.myId + " am helping to join " + ch.id);
        this.peerJoined(ch);
        this.checkMembers(ch, members);
    };
    FullMesh.prototype.initIntermediary = function (ch) {
        this.intermediaryChannel = ch;
    };
    FullMesh.prototype.initJoining = function (ch) {
        log.info("FULL_MESH: I:" + this.wc.myId + " am joining with help of " + ch.id);
        this.peerJoined(ch);
        this.joinAttempts = 0;
        this.intermediaryChannel = ch;
    };
    FullMesh.prototype.send = function (msg) {
        var bytes = this.wc.encode(msg);
        try {
            for (var _a = __values(this.channels), _b = _a.next(); !_b.done; _b = _a.next()) {
                var ch = _b.value;
                ch.send(bytes);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_1) throw e_1.error; }
        }
        var e_1, _c;
    };
    FullMesh.prototype.forward = function (msg) { };
    FullMesh.prototype.sendTo = function (msg) {
        var bytes = this.wc.encode(msg);
        try {
            for (var _a = __values(this.channels), _b = _a.next(); !_b.done; _b = _a.next()) {
                var ch = _b.value;
                if (ch.id === msg.recipientId) {
                    ch.send(bytes);
                    return;
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_2) throw e_2.error; }
        }
        if (this.intermediaryChannel) {
            this.intermediaryChannel.send((bytes));
            return;
        }
        else {
            try {
                for (var _d = __values(this.jps), _e = _d.next(); !_e.done; _e = _d.next()) {
                    var _f = __read(_e.value, 2), id = _f[0], ch = _f[1];
                    if (id === msg.recipientId) {
                        ch.send((bytes));
                        return;
                    }
                }
            }
            catch (e_3_1) { e_3 = { error: e_3_1 }; }
            finally {
                try {
                    if (_e && !_e.done && (_g = _d.return)) _g.call(_d);
                }
                finally { if (e_3) throw e_3.error; }
            }
        }
        console.warn(this.wc.myId + ' The recipient could not be found', msg.recipientId);
        var e_2, _c, e_3, _g;
    };
    FullMesh.prototype.forwardTo = function (msg) {
        this.sendTo(msg);
    };
    FullMesh.prototype.leave = function () {
        try {
            for (var _a = __values(this.channels), _b = _a.next(); !_b.done; _b = _a.next()) {
                var ch = _b.value;
                ch.close();
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_4) throw e_4.error; }
        }
        this.jps = new Map();
        this.joinAttempts = 0;
        this.intermediaryChannel = undefined;
        var e_4, _c;
    };
    FullMesh.prototype.onChannelClose = function (event, channel) {
        if (channel === this.intermediaryChannel) {
            this.leave();
            this.wc.joinSubject.next(new Error("Intermediary channel closed: " + event.type));
        }
        if (this.channels.delete(channel)) {
            this.wc.onMemberLeaveProxy(channel.id);
        }
    };
    FullMesh.prototype.onChannelError = function (evt, channel) {
        console.warn("Channel error: " + evt.type);
    };
    FullMesh.prototype.handleSvcMsg = function (_a) {
        var _this = this;
        var channel = _a.channel, senderId = _a.senderId, recipientId = _a.recipientId, msg = _a.msg;
        switch (msg.type) {
            case 'connectTo': {
                // Filter only missing peers
                var missingPeers = msg.connectTo.members.filter(function (id) { return id !== _this.wc.myId && !_this.wc.members.includes(id); });
                // Establish connection to the missing peers
                var misssingConnections = [];
                var _loop_1 = function (id) {
                    misssingConnections[misssingConnections.length] = new Promise(function (resolve) {
                        _this.wc.channelBuilder.connectTo(id)
                            .then(function (ch) {
                            _this.peerJoined(ch);
                            resolve();
                        })
                            .catch(function (err) {
                            console.warn(_this.wc.myId + ' failed to connect to ' + id, err.message);
                            resolve();
                        });
                    });
                };
                try {
                    for (var missingPeers_1 = __values(missingPeers), missingPeers_1_1 = missingPeers_1.next(); !missingPeers_1_1.done; missingPeers_1_1 = missingPeers_1.next()) {
                        var id = missingPeers_1_1.value;
                        _loop_1(id);
                    }
                }
                catch (e_5_1) { e_5 = { error: e_5_1 }; }
                finally {
                    try {
                        if (missingPeers_1_1 && !missingPeers_1_1.done && (_b = missingPeers_1.return)) _b.call(missingPeers_1);
                    }
                    finally { if (e_5) throw e_5.error; }
                }
                // Notify the intermediary peer about your members
                Promise.all(misssingConnections).then(function () {
                    var send = function () { return channel.send(_this.wc.encode({
                        recipientId: channel.id,
                        content: _super.prototype.encode.call(_this, { connectedTo: { members: _this.wc.members } }),
                    })); };
                    if (_this.joinAttempts === MAX_JOIN_ATTEMPTS) {
                        _this.leave();
                        _this.wc.joinSubject.next(new Error('Failed to join: maximum join attempts has reached'));
                        return;
                    }
                    if (_this.joinAttempts === 0) {
                        send();
                    }
                    else {
                        log.info("FULL_MESH: I:" + _this.wc.myId + " will resend my members " + _this.joinAttempts + " time(s)", {
                            myMembers: _this.wc.members,
                            intermediaryMembers: msg.connectTo.members,
                        });
                        setTimeout(function () { return send(); }, 200 + 100 * Math.random());
                    }
                    _this.joinAttempts++;
                });
                break;
            }
            case 'connectedTo': {
                this.checkMembers(channel, msg.connectedTo.members);
                break;
            }
            case 'joiningPeerId': {
                if (msg.joiningPeerId !== this.wc.myId && !this.wc.members.includes(msg.joiningPeerId)) {
                    this.jps.set(msg.joiningPeerId, channel);
                }
                break;
            }
            case 'joinSucceed': {
                this.intermediaryChannel = undefined;
                this.wc.joinSubject.next();
                break;
            }
        }
        var e_5, _b;
    };
    FullMesh.prototype.checkMembers = function (ch, members) {
        var _this = this;
        // Joining succeed if the joining peer and his intermediary peer
        // have same members (excludings themselves)
        if (this.wc.members.length === members.length && members.every(function (id) { return _this.wc.members.includes(id); })) {
            ch.send(this.wc.encode({
                recipientId: ch.id,
                content: this.joinSucceedContent,
            }));
            return;
        }
        // Joining did not finish, resend my members to the joining peer
        this.wc.sendProxy({ content: _super.prototype.encode.call(this, { joiningPeerId: ch.id }) });
        ch.send(this.wc.encode({
            recipientId: ch.id,
            content: _super.prototype.encode.call(this, { connectTo: { members: this.wc.members } }),
        }));
    };
    FullMesh.prototype.peerJoined = function (ch) {
        try {
            for (var _a = __values(this.channels), _b = _a.next(); !_b.done; _b = _a.next()) {
                var c = _b.value;
                if (c.id === ch.id) {
                    c.closeQuietly();
                    this.channels.delete(c);
                    this.channels.add(ch);
                    this.jps.delete(ch.id);
                    return;
                }
            }
        }
        catch (e_6_1) { e_6 = { error: e_6_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_6) throw e_6.error; }
        }
        this.channels.add(ch);
        this.wc.onMemberJoinProxy(ch.id);
        this.jps.delete(ch.id);
        var e_6, _c;
    };
    return FullMesh;
}(Service));
export { FullMesh };
